#ifndef _PARALLELS_CPP_
#define _PARALLELS_CPP_

#include <iostream.h>
#include <stdlib.h>

#include <qpainter.h>

#include <reference.h>
#include <table.h>
#include <event.h>
#include <part.h>
#include <track.h>
#include <scoreTrack.h>
#include <position.h>
#include <song.h>
#include <note.h>
#include <midiEvent.h>
#include <stdlib.h>
#include <songIterator.h>
#include <prPartEditor.h>
#include <container.h>

#include "parallels.h"

extern Song * sonG;

#define PARALLELS_CONTEXT Addon::TREAT_SONG + Addon::MARKING

extern "C" {
  Addon * parallels(Element * target) { return new Parallels(target); }
  const char  * parallels_name() { return "parallels"; }
  const char  * parallels_category() { return "harmony"; }
  int     parallels_context() { return PARALLELS_CONTEXT; }
}


Parallels::Parallels(Element * target) : Addon("find parallels", target, !TO_BE_UNDONE, PARALLELS_CONTEXT) {
  WEAK_PARALLEL   = 1;
  STRONG_PARALLEL = 2;
  INTERVAL_SIZE   = 4;
  run();
}

Parallels::~Parallels() {
}


int sgn(int i) {
  if (i==0) return 0;
  else if (i>0) return 1;
  else return -1;
}

void Parallels::song(Song * _song) {
  int ntracks = 0;
  ntracks = _song->size();

  _track = new ScoreTrack();
  _track->setName("* parallels...");
  _part = new Part(_track);
  _track->add(_part);
  _song->add(_track);
  long pp = 0;

  int maxvoices = ntracks*10;
  int pitches[maxvoices]; // in here we put the pitches to compare!
  int tendency[maxvoices]; // in here we put the tangent slope prev -> note
  int lengths[maxvoices];
  int count = 0;

  SongIterator * si = new SongIterator(_song, _song->left(), _song->right());

  Event * event = 0;
  Note * note   = 0;
  Track * tr    = 0;
  Note * prevnote = 0;
  int pitchDiff = 0;
  int parallel = 0;

  while ( !si->done() ) {
    count = 0;
    for (int t=0; t<ntracks; t++) {
      tr = si->track(t);
      while (si->startsAt(t,pp)) {
	if (!si->mute(t)) {
	  event = (Event*) (*si)[t];
	  if (event->isA()==NOTE) {
	    note = (Note*) event;
	    // alsa->playNote(0, aiterator->channel(t), tr->pitch(note), tr->vel(note), pp + tr->delay(), note->duration());
	    if (count<maxvoices) {
	      prevnote = (Note*) tr->prev(note);
	      if (prevnote!=0) tendency[count] = note->pitch() - prevnote->pitch(); else tendency[count] = 0;
	      lengths[count] = note->duration();
	      pitches[count++] = note->pitch();
	    } else {
	      cout << "ERROR: too many voices!" << endl;
	    }
	  }
	}
	si->increment(t);
      }
    }

    for (int i=0;i<count;i++) { // check for parallels:
      for (int k=0;k<i;k++) {
	pitchDiff = abs(pitches[i] - pitches[k]);
	parallel = (tendency[i]==tendency[k]) ? 1 : ( sgn(tendency[i])==sgn(tendency[k]) ? 2 : 0 ); // 0: nothing, 1: parallel, 2: verdeckt parallel
	// if (pitches[i] == pitches[k]) cout << "primes at " << pp << endl;
	if (pitchDiff==7) {
	  if (parallel==1) add("qp",pp,pitches[i],pitches[k],lengths[i],lengths[k],parallel); // quint parallels
	  else if (parallel==2) add("vqp",pp,pitches[i],pitches[k],lengths[i],lengths[k],parallel); // verdeckte quint parallels
	}
	if (pitchDiff==12) {
	  if (parallel==1) add("op",pp,pitches[i],pitches[k],lengths[i],lengths[k],parallel); // octave parallels
	  else if (parallel==2) add("vop",pp,pitches[i],pitches[k],lengths[i],lengths[k],parallel); // verdeckte octave parallels
	}
      }
    }

    pp++;
  }

}

void Parallels::add(const char * , long pos, int p1, int p2, int len1, int len2, int parallel) {
  _part->add(new Note(p1,87,len1,Position(pos),0,0,parallel-2));
  _part->add(new Note(p2,87,len2,Position(pos),0,0,parallel-2));
}


void Parallels::undo() {
}

void Parallels::redo() {
}

void Parallels::setMarks(PrPartEditor * ed) {
  Position left = ed->left();
  Position right = ed->right();



  SongIterator * si = new SongIterator(sonG, left, right);
  Event * ev=0;
  Event * ev1=0;
  Event * ev_old=0;
  Event * ev1_old=0;
  long pp = left.ticks();
  int sz = sonG->size();
  int pi=0;
  int pi1=0;
  int pi_old=0;
  int pi1_old=0;
  int diff=0;
  int move=0;
  int move1=0;
  Part * pt;
  Part * pt1;

  //
  // important: Make sure these are not marked, since they may not have been displayed and still be marked!!!
  //
  SongIterator * ci = new SongIterator(sonG, left, right);
  while (!ci->done()) {
    for (int t=0; t<sz; t++) {
      pt = ci->part(t);
      ev = (Event*) (*ci)[t];
      if (ev!=0) ev->unmark();
      ci->increment(t);
    }
  }

  //
  // loop and mark
  //
  while (!si->done()) {
    for (int t=0; t<sz; t++) {
      pt = si->part(t);
      while (si->startsAt(t,pp)) {
	ev = (Event*) (*si)[t];
	ev_old = (Event*) pt->prev(ev);
	if (ev!=0 && ev->isA()==NOTE) {
	  int kind = 0;
	  pi  = ((Note*)ev)->pitch();
	  if (ev_old!=0) pi_old = ((Note*)ev_old)->pitch();
	  if (pt->start(ev) > pt->end(ev_old)+2) ev_old = 0; // no parallels after breaks
	  if (ev_old!=0 && abs(pi_old-pi)>9 && pi_old>0) {
	    kind = kind | INTERVAL_SIZE; // interval too large
	    ev->mark(this, Container(kind));
	  }
	  for (int tt=t+1; tt<sz; tt++) {
	    pt1 = si->part(tt);
	    ev1 = (Event*) (*si)[tt];
	    ev1_old = (Event*) pt1->prev(ev1);
	    if (ev1!=0 && ev1->isA()==NOTE) {
	      pi1 = ((Note*)ev1)->pitch();
	      if (ev1_old!=0) pi1_old = ((Note*)ev1_old)->pitch();
	      if (pt->start(ev1) > pt->end(ev1_old)+2) ev1_old = 0; // no parallels after breaks
	      if (ev_old!=0 && ev1_old!=0 && !si->mute(t) && !si->mute(tt) && pt->start(ev)==pt->start(ev1)) {
		diff = abs(pi-pi1);
		move  = pi-pi_old;
		move1 = pi1-pi1_old;
		if ( (diff==0 || diff==7 || diff==12) && move*move1>0 ) {
		  kind = kind | ( move==move1 ? STRONG_PARALLEL : WEAK_PARALLEL );
		  ev->mark(this, Container(kind));
		  ev1->mark(this, Container(kind));
		}
	      }
	    }
	  }
	}
	
	si->increment(t);
      }
    }
    pp++;
  }
  
}


void Parallels::mark(Event * ev, int x, int y) {
  QPainter * _painter = (QPainter*) parameter();
  
  if (_painter!=0) {
    int kind = ((int) ev->parameter());
    
    if (kind==WEAK_PARALLEL || kind==STRONG_PARALLEL) {
      
      //
      // octave or quint parallels
      //
      
      if (kind==WEAK_PARALLEL)
	_painter->setPen(Qt::darkMagenta);
      else
	_painter->setPen(Qt::red);
      _painter->drawText(x+10, y-4, "||");
      _painter->drawText(x+11, y-4, "||");
    } else if (kind==INTERVAL_SIZE) {
      
      //
      // the interval is too large
      //
      
      _painter->setPen(Qt::red);
      _painter->drawText(x+10, y-4, "Int^");
    }
    
  }
}





#endif
