#ifndef _MARKER_H_
#define _MARKER_H_

#include "position.h"
#include "event.h"
#include "container.h"

class PrPartEditor;

/**
 * The Marker interface provides marking functionality to an addon: Setting an addon's context to Addon::MARKING adds a new entry in all editors' view menu.
 * Having this entry activated, setMarks() will be called before the content is drawn, providing an opportunity to mark certain notes.
 * While the content is drawn, mark() is called for each marked event.
**/
class Marker
{
 private:
  Container _parameter;

 public:
  Marker();
  virtual ~Marker();
  
  /**
   * Implement setMarks() by iterating through the song covering the range from left to right (obtained from the editor-parameter).
   * For each event ev to be marked, call ev->mark(this) in order to let the event know the marker to call during presentation.
   **/
  virtual void setMarks(PrPartEditor*);

  /**
   * mark() is called for each marked event during presentation. x and y are the (abstract) coordinates the event will be painted.
  **/
  virtual void mark(Event * ev, int x, int y);

  /**
   * This can be used to provide an extra parameter. This is already implemented.
   **/
  void setContainer(Container c);
  
  Container container();
  void * parameter();
};

#endif
