/***************************************************************************
                          mixer.cpp  -  Mixer window
                             -------------------
    begin                : Sun Jan  4 17:43:34 CET 2004
    copyright            : (C) 2004 by Morten Hustveit
    email                : mortehu@ifi.uio.no
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qlabel.h>
#include <qlayout.h>
#include <qslider.h>

#include <kdebug.h>

#include "configuration.h"
#include "sanetrackerdoc.h"
#include "mixer.h"

Mixer::Mixer(SanetrackerDoc& doc, const Configuration& config)
  : QWidget(0, "Mixer"), 
    doc(doc),
    config(config)
{
  setCaption("Mixer - Sanetracker");
  setGeometry(x(), y(), 480, height());

  QGridLayout* layout = new QGridLayout(this, doc.size(), 2);
  layout->setColSpacing(0, 100);

  sliders = new QSlider*[doc.size()];

  for(uint i = 0; i < doc.size(); ++i)
  {
    QLabel* label = new QLabel(doc[i].name(), this);
    sliders[i] = new QSlider(0, 127, 16, doc[i].volume(), Qt::Horizontal, this);

    if(i & 1)
    {
      label->setPaletteBackgroundColor(
        label->paletteBackgroundColor().light(110));
      sliders[i]->setPaletteBackgroundColor(
        sliders[i]->paletteBackgroundColor().light(110));
    }

    connect(sliders[i], SIGNAL(valueChanged(int)), this, SLOT(sliderMoved()));

    layout->addWidget(label, i, 0);
    layout->addWidget(sliders[i], i, 1);
  }
}

Mixer::~Mixer()
{
  kdDebug() << "Destroyed." << endl;
}

void Mixer::sliderMoved()
{
  for(uint i = 0; i < doc.size(); ++i)
    doc[i].setVolume(sliders[i]->value());

  doc.setModified();
}

