/***************************************************************************
                          sanetracker.h  -  description
                             -------------------
    begin                : Sat Jun  8 15:33:59 CEST 2002
    copyright            : (C) 2002 by Morten Hustveit
    email                : mortehu@ifi.uio.no
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef SANETRACKER_H
#define SANETRACKER_H


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

// include files for Qt

// include files for KDE
#include <kapp.h>
#include <kmainwindow.h>
#include <kaccel.h>
#include <kaction.h>
#include <kurl.h>

#include "configuration.h"

// forward declaration of the Sanetracker classes
class SanetrackerDoc;
class SanetrackerView;
class MIDIDriver;
class Scheduler;
class KIntNumInput;

class SanetrackerApp : public KMainWindow
{
Q_OBJECT

friend class SanetrackerView;
friend class Configuration;

public:

  SanetrackerApp(QWidget* parent=0, const char* name=0);
  ~SanetrackerApp();

  void openDocumentFile(const KURL& url=0);

  SanetrackerDoc* getDocument() const;
  SanetrackerView* getView() const;
  MIDIDriver* getMIDIDriver() const;

signals:

  void keyPressed(uint note, uint velocity);
  void keyReleased(uint note, uint velocity);

protected:

  bool event(QEvent*);

  void initActions();
  void initDocument();
  void initView();

  void reconfigure();

  virtual void saveProperties(KConfig *_cfg);
  virtual void readProperties(KConfig *_cfg);

public slots:

  void slotSettingsConfigure();
  void slotFileNew();
  void slotFileOpen();
  void slotFileOpenRecent(const KURL& url);
  void slotFileSave();
  void slotFileSaveAs();
  void slotFileQuit();
  void slotEditCut();
  void slotEditCopy();
  void slotEditPaste();
  void slotViewToolBar();
  void slotViewStatusBar();
  void slotStatusMsg(const QString &text);
  void slotModified(bool modified);
  void slotOpenMixer();

private:

  SanetrackerView* view;
  SanetrackerDoc* doc;

  // KAction pointers to enable/disable actions
  KAction* settingsConfigure;
  KAction* fileNew;
  KAction* fileOpen;
  KRecentFilesAction* fileOpenRecent;
  KAction* fileSave;
  KAction* fileSaveAs;
  KAction* fileQuit;
  KAction* editCut;
  KAction* editCopy;
  KAction* editPaste;
  KToggleAction* viewToolBar;
  KToggleAction* viewStatusBar;
  KAction* openMixer;

  MIDIDriver* midiDriver;
  Scheduler* scheduler;
  Configuration configuration;
  QWidget* mixer;

  uint octave;

  bool saveModified();
};

#endif // SANETRACKER_H
