#ifndef ARTSTRACKER_MAIN_H
#define ARTSTRACKER_MAIN_H
#include <kapp.h>
#include <klocale.h>
#include <kaboutdata.h>
#include <kcmdlineargs.h>
#include <qwidget.h>
#include <qpainter.h>
#include <qlayout.h>
#include <qscrollbar.h>
#include <qfont.h>
#include <queue>
#include "artstracker.h"
#include "artsmodules.h"
#include "artsmidi.h"

using namespace Arts::Tracker;
using namespace Arts;
using namespace std;

class ArtsTrackerWidget : public QWidget {
	Q_OBJECT
protected:
	Song song;
	Pattern p;
	MidiClient midiClient;
	MidiPort   midiPort;
	QFont font;
	QScrollBar *hbar, *vbar;

	int voffset, hoffset;
	int tracks, notes, cursorTrack, cursorNote, cursorOffset, cursorPattern, cursorSkip;
	
	int noteHeight, trackWidth;

	int octave;
	int instrument;
	int asPitch ( QKeyEvent* event );
public:
	ArtsTrackerWidget(QWidget *parent, QScrollBar *hbar, QScrollBar *vbar);
	void paintEvent(QPaintEvent *event);
	void paintNotes(int startNote, int startTrack, int endNote, int endTrack,
			bool eraseBorder = false, QRect *rect = 0);
	void keyPressEvent ( QKeyEvent* event );
	void keyReleaseEvent ( QKeyEvent* event );
	void cursorMove(int deltax, int deltay);
	void calcSizes();

	MidiPort port() { return midiPort; }
	Pattern pattern() { return p; }
	Song getSong() { return song; }

	void redrawNotes();

signals:
	void togglePlay();
	void octaveChanged(int newOctave);
	void instrumentChanged(int newInstrument);
	void patternChanged(int newPattern);

public slots:
	void changeHBar(int newvalue);
	void changeVBar(int newvalue);
	void setCursorNote(int);
	void setInstrument(int newInstrument);
	void setOctave(int newOctave);
	void setPattern(int newPattern);
	void songChanged();
};

class SongPlayer : public QObject {
	Q_OBJECT
protected:
	Song song;
	MidiPort midiPort;
	TimeStamp playTime, midiTime;
	QTimer *playTimer;
	bool playing;
	int currentPosition;
	int currentSongPosition;
	vector<int> activeNote;
	vector<int> activeInstrument;
	queue<int> playQueue;

public:
	SongPlayer(QObject *parent, Song s, MidiPort port);
	~SongPlayer();
	
	bool isPlaying();

signals:
	void playNote(int);
	void playPattern(int);

public slots:
	void togglePlay();
	void timerTick();
	void stopAllNotes();
};
#endif /* ARTSTRACKER_MAIN_H */
