#include "trackerview.h"
#include "songcontrol.h"
#include <qspinbox.h>
#include <kfiledialog.h>
#include <fstream>

using namespace std;

TrackerView::TrackerView(QWidget *parent) : QWidget(parent)
{
	QGridLayout *l = new QGridLayout(this,3,2);
	songControl = new SongControl(this);
	vbar = new QScrollBar(QScrollBar::Vertical,this);
	hbar = new QScrollBar(QScrollBar::Horizontal,this);
	twidget = new ArtsTrackerWidget(this,hbar,vbar);
	pp = new SongPlayer(this, twidget->getSong(), twidget->port());

	/*******************
	 *   songc         *
	 *******************
	 *  twidget * vbar *
	 *******************
	 * hbar     *      *
	 *******************/
	l->addMultiCellWidget(songControl,0,0,0,1);
	l->addWidget(twidget ,1,0);
	l->addWidget(vbar,1,1);
	l->addWidget(hbar,2,0);

	twidget->show();
	twidget->setFocus();
	hbar->show();
	vbar->show();

	connect(twidget,SIGNAL(togglePlay()),pp,SLOT(togglePlay()));
	connect(pp,SIGNAL(playNote(int)),twidget,SLOT(setCursorNote(int)));
	connect(pp,SIGNAL(playPattern(int)),twidget,SLOT(setPattern(int)));
	connect(hbar,SIGNAL(valueChanged(int)), twidget, SLOT(changeHBar(int)));
	connect(vbar,SIGNAL(valueChanged(int)), twidget, SLOT(changeVBar(int)));

	connect(songControl->instrument, SIGNAL(valueChanged(int)),
				twidget, SLOT(setInstrument(int)));
	connect(twidget, SIGNAL(instrumentChanged(int)), songControl->instrument,
	            SLOT(setValue(int)));

	connect(songControl->octave, SIGNAL(valueChanged(int)),
				twidget, SLOT(setOctave(int)));
	connect(twidget, SIGNAL(octaveChanged(int)), songControl->octave,
	            SLOT(setValue(int)));

	connect(songControl->pattern, SIGNAL(valueChanged(int)),
				twidget, SLOT(setPattern(int)));
	connect(songControl->pattern, SIGNAL(valueChanged(int)),
				this, SLOT(setPattern(int)));
	connect(twidget, SIGNAL(patternChanged(int)), songControl->pattern,
	            SLOT(setValue(int)));

/*
	connect(twidget, SIGNAL(songPatternsChanged()), this,
				SLOT(updateSongPatterns()));
*/
	connect(songControl->songLength, SIGNAL(valueChanged(int)),
				this, SLOT(setSongLength(int)));
	connect(songControl->songPos, SIGNAL(valueChanged(int)),
				this, SLOT(updateSongPatterns()));

	songControl->octave->setValue(5);
	songControl->songLength->setValue(1);
	l->activate();
}

void TrackerView::file_new()
{
	twidget->getSong().clear();
	twidget->songChanged();
}

void TrackerView::file_open()
{
	QString filename = KFileDialog::getOpenFileName(0,"*.at",this);
	ifstream infile(filename.ascii());
	string line;
	vector<string> strseq;

	while(getline(infile,line))
	strseq.push_back(line);

	twidget->getSong().loadFromList(strseq);
	twidget->songChanged();
}

void TrackerView::file_save()
{
	QString filename = KFileDialog::getSaveFileName(0,"*.at",this);
	vector<string> *savePtr = twidget->getSong().saveToList();
	vector<string>& list = *savePtr;

	FILE *file = fopen(filename.ascii(),"w");

	vector<string>::iterator i;

	for(i=list.begin();i != list.end(); i++)
		fprintf(file,"%s\n",(*i).c_str());
	fclose(file);

	delete savePtr;
}

void TrackerView::file_play_song()
{
	pp->togglePlay();
}

void TrackerView::file_stop_all_notes()
{
	pp->stopAllNotes();
}

void TrackerView::updateSongPatterns()
{
	songControl->patterns->clear();

	Song song = twidget->getSong();
	for(int i=0;i<song.songLength();i++)
	{
		QString text;
		text.sprintf("%02x",song.getSongPattern(i));
		songControl->patterns->insertItem(text);
	}
	songControl->patterns->setSelected(songControl->songPos->value(),true);
}

void TrackerView::setSongLength(int newSongLength)
{
	twidget->getSong().songLength(newSongLength);
	songControl->songPos->setRange(0,newSongLength-1);
	updateSongPatterns();
}

void TrackerView::setPattern(int newSongPattern)
{
	if(pp->isPlaying()) return;

	Song song = twidget->getSong();
	int pos = songControl->songPos->value();
	int len = song.songLength();

	if(pos >= len || pos < 0) return;

	song.setSongPattern(pos,newSongPattern);
	updateSongPatterns();
}
